#include "histogram.h"

#include <QDebug>
#include <QPainter>

#include <cmath>

Histogram::Histogram(QImage* image)
{
    R = new QHash<int, int>;
    G = new QHash<int, int>;
    B = new QHash<int, int>;
    L = new QHash<int, int>;
    generate(image);
}

Histogram::~Histogram()
{
    delete R;
    delete G;
    delete B;
    delete L;
}

void Histogram::incrementChannel(QHash<int, int>* channel, int pixelValue)
{
    channel->insert(pixelValue, channel->value(pixelValue)+1);
}

void Histogram::generate(QImage* image)
{
    int width = image->width();
    int height = image->height();

    for (int x=0; x<width; x++)
        for (int y=0; y<height; y++) {
              QRgb pixel = image->pixel(x,y);
              incrementChannel(R, qRed(pixel));
              incrementChannel(G, qGreen(pixel));
              incrementChannel(B, qBlue(pixel));
              incrementChannel(L, qGray(pixel));
        }

}

int Histogram::getMax(Channel c) {
        std::list<int> list = this->get(c)->keys().toStdList();
        list.sort();
        return list.back();
}

int Histogram::getMin(Channel c) {
    std::list<int> list = this->get(c)->keys().toStdList();
    list.sort();
    return list.front();
}

int Histogram::maximumValue(Channel selectedChannel = RGB)
{
    if (selectedChannel == RGB)
        return maximumRGBValue();
    else
        return maximumSingleChannelValue(selectedChannel);
}


int Histogram::maximumSingleChannelValue(Channel selectedChannel)
{
    std::list<int> c = this->get(selectedChannel)->values().toStdList();
    c.sort();
    if (c.size() > 0) {
        return c.back();
    } else return 0;
}

int Histogram::maximumRGBValue()
{
    int maxR = maximumValue(RChannel);
    int maxG = maximumValue(GChannel);
    int maxB = maximumValue(BChannel);

    return std::max({maxR, maxG, maxB});
}


/** Returns a pointer to the given channel QHash<int, int> */
QHash<int, int>* Histogram::get(Channel channel = LChannel)
{
    if (channel==LChannel) return L;
    if (channel==RChannel) return R;
    if (channel==GChannel) return G;
    if (channel==BChannel) return B;

    return NULL;
}

/**
 *  Returns a 255 by 100 QImage containing a Histogram for the given channel.
 *  The background is transparent (Alpha 0, RGB=255) */
QImage Histogram::getImage(Channel channel = LChannel, QBrush pen = Qt::gray)
{
    QImage histImage(255, 100, QImage::Format_ARGB32);
    histImage.fill(0);
    QPainter painter(&histImage);
    painter.setBrush(Qt::transparent); 
    painter.setPen(Qt::transparent);
    painter.drawRect(0,0,255,100);

    int maximum = (channel == LChannel ? maximumValue(LChannel) :  maximumValue(RGB));
    float ratio = 100.0/float(maximum);

    painter.setBrush(pen);
    painter.setPen(pen.color());

    int h;
    QHash<int, int>* hist = get(channel);
    QHash<int, int>::const_iterator cit = hist->begin();

    while (cit != hist->end())
    {
        h = 100 - floor(ratio*cit.value());
        painter.drawLine(cit.key(), h, cit.key(), 100);
        ++cit;
    }

    return histImage;
}
