#include "blur_gaussian.h"

BlurGaussian::BlurGaussian(PNM* img) :
    Convolution(img)
{
}

BlurGaussian::BlurGaussian(PNM* img, ImageViewer* iv) :
    Convolution(img, iv)
{
}

PNM* BlurGaussian::transform()
{
    emit message("Blurring...");

    int size = getParameter("size").toInt();
    radius = (size/2)+1;
    sigma = getParameter("sigma").toDouble();

    return convolute(getMask(size, Normalize), RepeatEdge);
}

math::matrix<float> BlurGaussian::getMask(int size, Mode)
{
    math::matrix<float> mask(size, size);
    for (int x=0;x<size;++x) {
        for (int y=0;y<size;++y) {
            mask[x][y] = getGauss(x - int(size / 2), y - int(size / 2), sigma);
        }
    }
    return mask;
}

float BlurGaussian::getGauss(int x, int y, float sigma)
{    
    return exp(-((pow(x, 2.0) + pow(y, 2.0)) / (2 * pow(sigma, 2.0))))
            /
            (2 * M_PI * pow(sigma, 2.0));
}

