#include "convolution.h"

/** Overloaded constructor */
Convolution::Convolution(PNM* img) :
    Transformation(img)
{
}

Convolution::Convolution(PNM* img, ImageViewer* iv) :
    Transformation(img, iv)
{
}

/** Returns a convoluted form of the image */
PNM* Convolution::transform()
{
    return convolute(getMask(3, Normalize), RepeatEdge);
}

math::matrix<float> Convolution::getMask(int size, Mode mode = Normalize) {
    math::matrix<float> mask(size, size);

        for (int i=0; i<size; ++i) {
            for (int j=0; j<size; ++j) {
                if (i / 2 == size / 2)
                    mask[i][j] = 1;
                else
                    mask[i][j] = 0;
            }
        }

        return mask;
}

PNM* Convolution::convolute(math::matrix<float> mask, Mode mode = RepeatEdge)
{
    int width  = image->width(),
        height = image->height();

    PNM* newImage = new PNM(width, height, image->format());

    float weight = sum(mask);

    for (int x=0; x<width; ++x) {
        for (int y=0; y<height; ++y) {

            math::matrix<float> window = getWindow(x, y, mask.colsize(), RChannel, mode);
            float summ = sum(join(window, mask));
            if (weight != 0) summ = summ / weight;
            if (summ < 0) summ = 0;
            else if (summ > 255) summ = 255;
            int r = summ;

            window = getWindow(x, y, mask.colsize(), GChannel, mode);
            summ = sum(join(window, mask));
            if (weight != 0) summ = summ / weight;
            if (summ < 0) summ = 0;
            else if (summ > 255) summ = 255;
            int g = summ;

            window = getWindow(x, y, mask.colsize(), BChannel, mode);
            summ = sum(join(window, mask));
            if (weight != 0) summ = summ / weight;
            if (summ < 0) summ = 0;
            else if (summ > 255) summ = 255;
            int b = summ;

            newImage->setPixel(x, y, QColor(r,g,b).rgb());
        }
    }

    return newImage;
}


const math::matrix<float> Convolution::join(math::matrix<float> A, math::matrix<float> B)
{
    int size = A.rowno();
    math::matrix<float> C(size, size);

    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
          C[i][j] = A[i][j] * B[i][j];
      }
    }

    return C;
}

const float Convolution::sum(const math::matrix<float> A)
{
    float sum = 0.0;
    int size = A.rowno();

    for (int i = 0; i < size; i++) {
        for (int j = 0; j < size; j++) {
           sum += A[i][j];
        }
    }

    return sum;
}

const math::matrix<float> Convolution::reflection(const math::matrix<float> A)
{
    int size = A.rowno();
    math::matrix<float> C(size, size);

    for (int i=0; i<size; ++i) {
        for (int j=0; j<size; ++j) {
            C[size - i][size - j] = A[i][j];
        }
    }

    return C;
}

