#include "correction.h"

Correction::Correction(PNM* img) :
    Transformation(img)
{
}

Correction::Correction(PNM* img, ImageViewer* iv) :
    Transformation(img, iv)
{
}

PNM* Correction::transform()
{
    float shift  = getParameter("shift").toFloat();
    float factor = getParameter("factor").toFloat();
    float gamma  = getParameter("gamma").toFloat();

    int width  = image->width();
    int height = image->height();

    PNM* newImage = new PNM(width, height, image->format());

    for (int i=0; i<256; ++i) {

        int newColor = pow(i, gamma);
        newColor = newColor * factor;
        newColor = newColor + shift;

        if (newColor > 255) newColor = 255;
        else if (newColor < 0) newColor = 0;

        LUT[i] = newColor;
    }

    for (int x=0; x<width; x++)
        for (int y=0; y<height; y++)
        {
            QRgb pixel = image->pixel(x,y);

            int red = LUT[qRed(pixel)];
            int green = LUT[qGreen(pixel)];
            int blue = LUT[qBlue(pixel)];

            newImage->setPixel(x,y, QColor(red,green,blue).rgb());
        }

    return newImage;
}

