#include "edge_canny.h"

#include "blur_gaussian.h"
#include "conversion_grayscale.h"
#include "edge_sobel.h"

EdgeCanny::EdgeCanny(PNM* img) :
    Convolution(img)
{
}

EdgeCanny::EdgeCanny(PNM* img, ImageViewer* iv) :
    Convolution(img, iv)
{
}

PNM* EdgeCanny::transform()
{
    int width  = image->width(),
        height = image->height();

    int upper_thresh = getParameter("upper_threshold").toInt(),
        lower_thresh = getParameter("lower_threshold").toInt();


    PNM* newImage = new PNM(width, height, QImage::Format_Grayscale8);

    //TO GRAY
    ConversionGrayscale gray(image);
    newImage = gray.transform();

    //BLUR
    BlurGaussian gauss(newImage);
    gauss.setParameter("size", 3);
    gauss.setParameter("sigma", 1.6);
    newImage = gauss.transform();

    //SOBEL DETECTION
    EdgeSobel sobel(newImage);
    math::matrix<float>* horizontal = sobel.rawHorizontalDetection();
    math::matrix<float>* vertical = sobel.rawVerticalDetection();


    math::matrix<float> strenghts(width, height);
    math::matrix<int> angles(width, height);

    for (int x=0;x<width;++x) {
        for (int y=0;y<height;++y) {
            float Gx = (*horizontal)[x][y];
            float Gy = (*vertical)[x][y];
            strenghts[x][y] = sqrt( pow(Gx,2.0) + pow(Gy,2.0) );
            float angle = (atan2(Gy,Gx)/3.14159) * 180.0;
            float a = 0;


            if (((angle < 22.5) && (angle > -22.5)) || (angle > 157.5) || (angle < -157.5))
               a = 0;
            else if (((angle > 22.5) && (angle < 67.5)) || ((angle < -112.5) && (angle > -157.5)))
               a = 1;
            else if (((angle > 67.5) && (angle < 112.5)) || ((angle < -67.5) && (angle > -112.5)))
               a = 2;
            else if (((angle > 112.5) && (angle < 157.5)) || ((angle < -22.5) && (angle > -67.5)))
               a = 3;

            angles[x][y] = a;
         }
    }

       for (int x=1;x<width-1;++x) {
           for (int y=1;y<height-1;++y) {
               int angle = angles[x][y];

               float n1 = 0;
               float n2 = 0;

               float n = strenghts[x][y];

               if (angle == 0) {
                   n1 = strenghts[x][y-1];
                   n2 = strenghts[x][y+1];
               } else if (angle == 1) {
                   n1 = strenghts[x-1][y-1];
                   n2 = strenghts[x+1][y+1];
               } else if (angle == 2) {
                   n1 = strenghts[x-1][y];
                   n2 = strenghts[x+1][y];
               } else if (angle == 3) {
                   n1 = strenghts[x+1][y+1];
                   n2 = strenghts[x-1][y-1];
               }

               if (n > n1 && n > n2 && n > upper_thresh / 3.33){
                   n = n > 255 ? 255 : n;
                   newImage->setPixel(x, y, QColor(n, n, n).rgb());
               } else
                   newImage->setPixel(x, y, QColor(0, 0, 0).rgb());
           }
       }

       int highThresh = (upper_thresh / 3.33) * 255 / 100;
       int lowThresh = lower_thresh * 255 / 100;

       unsigned int high = QColor(highThresh,highThresh,highThresh).rgb();
       unsigned int low = QColor(lowThresh,lowThresh,lowThresh).rgb();
       unsigned int weak = QColor(128,128,128).rgb();
       unsigned int strong = QColor(255,255,255).rgb();

       for (int x=1;x<width-1;++x) {
               for (int y=1;y<height-1;++y) {



                   if (newImage->pixel(x,y) >= high) {
                       newImage->setPixel(x, y, strong);
                   }
                   else if (newImage->pixel(x,y) < high && newImage->pixel(x,y) >= low) {
                       newImage->setPixel(x, y, weak);
                   }
                   else {
                       newImage->setPixel(x, y, QColor(0, 0, 0).rgb());
                   }

                   unsigned int white = QColor(255, 255, 255).rgb();
                   if (newImage->pixel(x, y) == weak) {
                        if (
                               (newImage->pixel(x+1, y-1) == white) ||
                               (newImage->pixel(x+1, y) == white) ||
                               (newImage->pixel(x+1, y+1) == white) ||
                               (newImage->pixel(x, y-1) == white) ||
                               (newImage->pixel(x, y+1) == white) ||
                               (newImage->pixel(x+1, y-1) == white) ||
                               (newImage->pixel(x-1, y-1) == white) ||
                               (newImage->pixel(x-1, y) == white) ||
                               (newImage->pixel(x-1, y+1) == white)
                            )
                       newImage->setPixel(x, y, QColor(255, 255, 255).rgb());
                   else
                        newImage->setPixel(x, y, QColor(0, 0, 0).rgb());
                   }




           }
       }


    return newImage;
}
