#include "edge_sobel.h"

EdgeSobel::EdgeSobel(PNM* img, ImageViewer* iv) :
    EdgeGradient(img, iv)
{
    prepareMatrices();
}

EdgeSobel::EdgeSobel(PNM* img) :
    EdgeGradient(img)
{
    prepareMatrices();
}

void EdgeSobel::prepareMatrices()
{
    float array_x[9] = {1,0,-1,2,0,-2,1,0,-1};
    float array_y[9] = {1,2,1,0,0,0,-1,-2,-1};

    g_x = math::matrix<float>(3,3, array_x);
    g_y = math::matrix<float>(3,3, array_y);
}

math::matrix<float>* EdgeSobel::rawHorizontalDetection()
{
    int width = image->width();
    int height = image->height();
    math::matrix<float>* x_gradient = new math::matrix<float>(width, height);

    PNM* newImage = convolute(g_x, RepeatEdge);

    for (int x=0;x<width;++x) {
        for (int y=0;y<height;++y) {
            (*x_gradient)[x][y] = qGray(newImage->pixel(x,y));
        }
    }

    delete newImage;
    return x_gradient;
}

math::matrix<float>* EdgeSobel::rawVerticalDetection()
{
    int width = image->width();
    int height = image->height();
    math::matrix<float>* y_gradient = new math::matrix<float>(width, height);


    PNM* newImage = convolute(g_y, RepeatEdge);

    for (int x=0;x<width;++x) {
        for (int y=0;y<height;++y) {
            (*y_gradient)[x][y] = qGray(newImage->pixel(x,y));
        }
    }

    delete newImage;
    return y_gradient;
}
