#include "histogram_equalization.h"

#include "../histogram.h"

HistogramEqualization::HistogramEqualization(PNM* img) :
    Transformation(img)
{
}

HistogramEqualization::HistogramEqualization(PNM* img, ImageViewer* iv) :
    Transformation(img, iv)
{
}

PNM* HistogramEqualization::transform()
{
    int width = image->width();
    int height = image->height();

    PNM* newImage = new PNM(width, height, image->format());

    std::list<int> rHash = image->getHistogram()->get(Histogram::RChannel)->values().toStdList();
    double rSum = std::accumulate(std::begin(rHash), std::end(rHash), 0.0);

    std::list<int> gHash = image->getHistogram()->get(Histogram::GChannel)->values().toStdList();
    double gSum = std::accumulate(std::begin(gHash), std::end(gHash), 0.0);

    std::list<int> bHash = image->getHistogram()->get(Histogram::BChannel)->values().toStdList();
    double bSum = std::accumulate(std::begin(bHash), std::end(bHash), 0.0);


     QHash<int, int>* rData = image->getHistogram()->get(Histogram::RChannel);
     QHash<int, int>* gData = image->getHistogram()->get(Histogram::GChannel);
     QHash<int, int>* bData = image->getHistogram()->get(Histogram::BChannel);

    int r[256];
    int g[256];
    int b[256];

    float rD = 0.0;
    float gD = 0.0;
    float bD = 0.0;
    for (int i=0;i<256; ++i)
    {
         rD += rData->value(i) / rSum;

         gD += gData->value(i) / gSum;


         bD += bData->value(i) / bSum;


        r[i] = rD * 255;
        g[i] = gD * 255;
        b[i] = bD * 255;
    }


    for (int x=0; x<width; ++x)
    {
        for (int y=0; y<height; ++y)
        {
            QRgb pixel = image->pixel(x,y);

            int red = r[qRed(pixel)];
            int green = g[qGreen(pixel)];
            int blue = b[qBlue(pixel)];

            newImage->setPixel(x,y, QColor(red,green,blue).rgb());
        }
    }

    return newImage;
}


