#include "histogram_stretching.h"

#include "../histogram.h"

HistogramStretching::HistogramStretching(PNM* img) :
    Transformation(img)
{
}

HistogramStretching::HistogramStretching(PNM* img, ImageViewer* iv) :
    Transformation(img, iv)
{
}

PNM* HistogramStretching::transform()
{
    int width  = image->width();
    int height = image->height();

    PNM* newImage = new PNM(width, height, image->format());




    int gMax = image->getHistogram()->getMax(Histogram::GChannel);
    int gMin = image->getHistogram()->getMin(Histogram::GChannel);
    int rMax = image->getHistogram()->getMax(Histogram::RChannel);
    int rMin = image->getHistogram()->getMin(Histogram::RChannel);
    int bMax = image->getHistogram()->getMax(Histogram::BChannel);
    int bMin = image->getHistogram()->getMin(Histogram::BChannel);


    for (int x=0; x<width; ++x)
    {
        for (int y=0; y<height; ++y)
        {
            QRgb pixel = image->pixel(x,y);

            int red = (255 * (qRed(pixel) - rMin)) / (rMax - rMin);
            int green = (255 * (qGreen(pixel) - gMin)) / (gMax - gMin);
            int blue = (255 * (qBlue(pixel) - bMin)) / (bMax - bMin);

            newImage->setPixel(x,y, QColor(red,green,blue).rgb());
        }
    }

    return newImage;
}


