#include "hough.h"

#include "conversion_grayscale.h"
#include "edge_laplacian.h"
#include <math.h>

Hough::Hough(PNM* img) :
    Transformation(img)
{
}

Hough::Hough(PNM* img, ImageViewer* super) :
    Transformation(img, super)
{
}

PNM* Hough::transform()
{   
    int thetaDensity = getParameter("theta_density").toInt();
    bool skip_edge = getParameter("skip_edge_detection").toBool();

    ConversionGrayscale gray(image);

    PNM* newImage = new PNM(image->width(), image->height(), QImage::Format_Grayscale8);
    newImage = gray.transform();

    if (skip_edge == false) {
        EdgeLaplacian laplacian(newImage);
        laplacian.setParameter("size", 5);
        newImage = laplacian.transform();
    }

    float diag = sqrt( pow(image->width(), 2) + pow(image->height(), 2) );
    float thetaSize = 180 * thetaDensity;

    int width = thetaSize;
    int height = (diag * 2) + 1;

    PNM* resultImage = new PNM(width, height, QImage::Format_Grayscale8);

    math::matrix<float> houghtMatrix(width, height, 0.0);
    math::matrix<float> theta(width, height);

    for (int x=0;x<image->width();++x) {
        for (int y=0;y<image->height();++y) {
            if (qGray(newImage->pixel(x,y)) > 0) {
                for (int k = 0; k < thetaSize; ++k) {
                    float theta = (k * M_PI) / (thetaDensity * 180.0);
                    float p = (x * cos(theta)) + (y * sin(theta));
                    int second = diag + p;
                    houghtMatrix[k][second] += 1;
                }

            }
        }
    }

    delete newImage;

    float maxHough = 255 / houghtMatrix.max();
    for (int x=0;x<width;++x) {
        for (int y=0;y<height;++y) {
            houghtMatrix[x][y] *= maxHough;
            resultImage->setPixel(x,
                                  y,
                                  QColor(houghtMatrix[x][y],
                                         houghtMatrix[x][y],
                                         houghtMatrix[x][y]
                                         ).rgb());
        }
    }

    return resultImage;

}
