#include "morphological_operator.h"

MorphologicalOperator::MorphologicalOperator(PNM* img) :
    Transformation(img)
{
}

MorphologicalOperator::MorphologicalOperator(PNM* img, ImageViewer* iv) :
    Transformation(img, iv)
{
}

// abstract
const int MorphologicalOperator::morph(math::matrix<float>, math::matrix<bool>)
{
    return 0;
}

math::matrix<bool> MorphologicalOperator::getSE(int size, SE shape)
{
    switch (shape)
    {
    case Square:    return seSquare(size);
    case Cross:     return seCross(size);
    case XCross:    return seXCross(size);
    case VLine:     return seVLine(size);
    case HLine:     return seHLine(size);
    default:        return seSquare(size);
    }
}


math::matrix<bool> MorphologicalOperator::seSquare(int size)
{
    math::matrix<bool> ret(size, size);

    for (int x=0;x<size;++x) for (int y=0;y<size;++y) ret[x][y] = true;

    return ret;
}

math::matrix<bool> MorphologicalOperator::seCross(int size)
{
    math::matrix<bool> ret(size, size);

    for (int x=0;x<size;++x) {
        for (int y=0;y<size;++y) {
            ret[x][y] = (x/2 == size/2 || y/2 == size/2) ? true : false;
        }
    }

    return ret;
}

math::matrix<bool> MorphologicalOperator::seXCross(int size)
{
    math::matrix<bool> ret(size, size, false);

    for (int i=0;i<size;++i) {
        ret[i][i] = true;
        ret[1][size-1-i] = true;
    }

    return ret;
}

math::matrix<bool> MorphologicalOperator::seVLine(int size)
{
    math::matrix<bool> ret(size, size);

    for (int x=0;x<size;++x) {
        for (int y=0;y<size;++y) {
             ret[x][y] = (x/2 == size/2) ? true : false;
        }
    }

    return ret;
}

math::matrix<bool> MorphologicalOperator::seHLine(int size)
{
    math::matrix<bool> ret(size, size);

    for (int x=0;x<size;++x) {
        for (int y=0;y<size;++y) {
             ret[x][y] = (y/2 == size/2) ? true : false;
        }
    }

    return ret;
}

PNM* MorphologicalOperator::transform()
{  
    int size  = getParameter("size").toInt();
    SE  shape = (MorphologicalOperator::SE) getParameter("shape").toInt();

    PNM* newImage = new PNM(image->width(), image->height(), QImage::Format_RGB32);

    for (int x=0;x<image->width();++x) {
            for (int y=0;y<image->height();++y) {

                math::matrix<float> windowR = getWindow(x, y, size, RChannel, RepeatEdge);
                int newR = morph(windowR, getSE(size, shape));

                math::matrix<float> windowG = getWindow(x, y, size, GChannel, RepeatEdge);
                int newG = morph(windowG, getSE(size, shape));

                math::matrix<float> windowB = getWindow(x, y, size, BChannel, RepeatEdge);
                int newB = morph(windowB, getSE(size, shape));

                newImage->setPixel(x,y, QColor(newR, newG, newB).rgb());
            }
        }

    return newImage;
}
