#include "noise_bilateral.h"

NoiseBilateral::NoiseBilateral(PNM* img) :
    Convolution(img)
{
}

NoiseBilateral::NoiseBilateral(PNM* img, ImageViewer* iv) :
    Convolution(img, iv)
{
}

PNM* NoiseBilateral::transform()
{
    int width  = image->width();
    int height = image->height();

    PNM* newImage = new PNM(width, height, image->format());

    sigma_d = getParameter("sigma_d").toInt();
    sigma_r = getParameter("sigma_r").toInt();
    radius = sigma_d * 2;

    for (int x=0;x<width;++x) {
        for (int y=0;y<height;++y) {
            int r = calcVal(x,y, RChannel);
            int g = calcVal(x,y, GChannel);
            int b = calcVal(x,y, BChannel);
            newImage->setPixel(x, y, QColor(r,g,b).rgb());
        }
    }

    return newImage;
}

int NoiseBilateral::calcVal(int x, int y, Channel channel)
{
    math::matrix<float> window = getWindow(x, y, radius, channel, RepeatEdge);

    int pixel = image->pixel(x,y);

    switch (channel) {
      case RChannel: pixel = qRed(pixel);
      case GChannel: pixel = qGreen(pixel);
      case BChannel: pixel = qBlue(pixel);
    }

    float sum1 = 0;
    float sum2 = 0;

    for (int i=0;i<radius;++i) {
        for (int j=0;j<radius;++j) {
            int pixelVal = window[i][j];
            QPoint point1((i - radius/2) + x, (j - radius/2) + y);
            QPoint point2(x,y);
            sum1 += pixelVal * colorCloseness(pixelVal, pixel) * spatialCloseness(point1, point2);
            sum2 += colorCloseness(pixelVal, pixel) * spatialCloseness(point1, point2);
        }
    }

    return sum2 != 0 ? sum1 / sum2 : 0;
}

float NoiseBilateral::colorCloseness(int val1, int val2)
{
    return exp( -(pow(val1 - val2, 2) / (2 * pow(sigma_r, 2))));
}

float NoiseBilateral::spatialCloseness(QPoint point1, QPoint point2)
{
    return exp(
                -(pow(point1.x() - point2.x(), 2) + pow(point1.y() - point2.y(), 2))
                /
                (2 * pow(sigma_d * 2, 2))
              );
}
