#include "noise_median.h"

NoiseMedian::NoiseMedian(PNM* img) :
    Convolution(img)
{
}

NoiseMedian::NoiseMedian(PNM* img, ImageViewer* iv) :
    Convolution(img, iv)
{
}

PNM* NoiseMedian::transform()
{
    int width  = image->width();
    int height = image->height();

    PNM* newImage = new PNM(width, height, image->format());

    for (int x=0;x<width;++x) {
            for (int y=0;y<height;++y) {
                int r = getMedian(x,y, RChannel);
                int g = getMedian(x,y, GChannel);
                int b = getMedian(x,y, BChannel);
                newImage->setPixel(x, y, QColor(r,g,b).rgb());
            }
        }

    return newImage;
}

int NoiseMedian::getMedian(int x, int y, Channel channel)
{
    int radius = getParameter("radius").toInt();
    radius = radius * 2 + 1;

    math::matrix<float> window = getWindow(x, y, radius, channel, RepeatEdge);
        int table_size = radius * radius;
        int table[table_size];

        int table_index = 0;

        for (int x=0;x<radius;++x) {
            for (int y=0;y<radius;++y) {
                table[table_index] = window[x][y];
                table_index++;
            }
        }

        std::sort(table, table + (table_size - 1));

        int median;

        if (table_size % 2 != 0)
            median = table[ table_size / 2 ];
        else
            median = (table[ (table_size - 1) / 2 ] + table[ table_size / 2 ]) / 2.0;

        return median;
}
